/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#ifndef CONF_H
#define CONF_H

#include <gtk/gtktooltips.h>
/* structures for representing logjam data. 
 * livejournal-specific structures are in livejournal.h.
 */ 
#include "config.h"
#include "livejournal.h"
#include "checkfriends.h"
#include "remote.h"

typedef struct {
	int x, y, width, height;
	int panedpos; /* optional; for windows with panes. */
} Geometry;

/* this should match the geometry_names[] array in conf_xml.c */
typedef enum {
	GEOM_MAIN,
	GEOM_LOGIN,
	GEOM_FRIENDS,
	GEOM_FRIENDGROUPS,
	GEOM_CONSOLE,
	GEOM_MANAGER,
	GEOM_CFFLOAT,
	GEOM_OFFLINE,
	GEOM_PREVIEW,
	GEOM_COUNT
} GeometryType;

typedef struct {
	gboolean revertusejournal;
	gboolean netdump;
	gboolean nofork;
	gboolean useproxy;
	gboolean useproxyauth;
#ifdef HAVE_GTKSPELL
	gboolean usespellcheck;
#endif
	gboolean autosave;
	gboolean cfautostart;
	gboolean cfusemask;
	gboolean cfdocklet;
	gboolean cffloat;
	gboolean cffloatraise;
	gboolean cffloat_decorate;
	gboolean friends_statsvis, friends_filtervis;
	gboolean allowmultipleinstances;
	gboolean smartquotes;
} Options;

enum {
	POSTMODE_GUI,
	POSTMODE_CMD
} PostmodeType;

typedef enum {
	MUSIC_SOURCE_XMMS,
	MUSIC_SOURCE_COMMAND,
	MUSIC_SOURCE_NONE
} MusicSource;

typedef struct {
	/* configuration file */
	GList *servers;
	Server *servercur;

	Geometry geometries[GEOM_COUNT];

	Options options;

	gchar *uifont;
#ifdef HAVE_GTKSPELL
	gchar *spell_language;
#endif
#ifndef G_OS_WIN32
	char *spawn_command;

	MusicSource music_source;
	char *music_command;

	char *proxy;
	char *proxyuser, *proxypass;
#endif

	Security defaultsecurity;

	gint cfuserinterval;
	gint cfthreshold;

	/* run-time settings. */
	gboolean loginok;
	char *usejournal;
	int postmode;
} Configuration;

typedef struct {
	gchar *programname;
	gchar *conf_dir;        /* may be null, which means <home>/.logjam/ */

	GtkTooltips *tooltips;
	GSList *secmgr_list;
	CFMgr *cfmgr;

	GList *quiet_dlgs;

	CFIndicator *cfi_float, *cfi_docklet;

	gboolean cli;           /* true if there's no gui */
	gboolean quiet;

	gboolean autolog;   /* log in as current user without prompting. */
	gboolean skiplogin; /* don't connect to server while changing users. */

	gboolean autosave_ok;      /* TRUE by default, disabled in cli mode */
	gboolean autoloaddraft_ok; /* TRUE by default, unset by -f and -e */
	gint autosave;          /* timeout id */

	LogJamRemote *remote;
} Application;

Server*  conf_server_by_name(Configuration *c, const char *servername);

extern Configuration conf;
extern Application app;
#define conf_cur_server() conf.servercur
#define conf_cur_user() conf_cur_server()->usercur

int conf_verify_dir(void);
void conf_make_path(char *file, char *buf, int len);

char* conf_make_curuser_path(const char *path);

void conf_verify_a_server_exists();

gboolean conf_rename_server(Server *server, const char *newname, GError **err);

#endif /* CONF_H */
