/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#ifndef __logjam_doc_h__
#define __logjam_doc_h__

#include "livejournal.h"
#include <gtk/gtktextbuffer.h>

typedef enum {
	ENTRY_DRAFT,
	ENTRY_NEW,
	ENTRY_SERVER
} EntryType;

typedef struct _LogJamDoc LogJamDoc;
typedef struct _LogJamDocClass LogJamDocClass;

GType      logjam_doc_get_type(void);
LogJamDoc* logjam_doc_new(void);

GtkTextBuffer* logjam_doc_get_text_buffer(LogJamDoc *doc);
char *         logjam_doc_get_title      (LogJamDoc *doc);
Entry*         logjam_doc_get_entry      (LogJamDoc *doc);

char *         logjam_doc_get_draftname  (LogJamDoc *doc);

/* these flags correspond to server-side capabilities;
 * any document can be saved with a file->save as sort of command. */
#define LOGJAM_DOC_CAN_DELETE (1<<0)
#define LOGJAM_DOC_CAN_SAVE   (1<<1)
#define LOGJAM_DOC_CAN_SUBMIT (1<<2)
int            logjam_doc_get_flags      (LogJamDoc *doc);

void           logjam_doc_set_dirty      (LogJamDoc *doc, gboolean dirty);
gboolean       logjam_doc_get_dirty      (LogJamDoc *doc);

Security logjam_doc_get_security(LogJamDoc *doc);
void logjam_doc_set_security(LogJamDoc *doc, Security *sec);
void logjam_doc_set_comments(LogJamDoc *doc, CommentsType type);
void logjam_doc_set_backdated(LogJamDoc *doc, gboolean backdated);
void logjam_doc_set_preformatted(LogJamDoc *doc, gboolean preformatted);
void logjam_doc_set_pickeyword(LogJamDoc *doc, const char *keyword);

gboolean logjam_doc_load_file (LogJamDoc *doc, const char *filename,
                               EntryFileType type, GError **err);
void     logjam_doc_load_draft(LogJamDoc *doc, Entry *entry);
void     logjam_doc_load_entry(LogJamDoc *doc, Entry *entry);

/* these are terrible names. */
gboolean logjam_doc_would_save_over_nonxml(LogJamDoc *doc);
gboolean logjam_doc_has_save_target(LogJamDoc *doc);

gboolean logjam_doc_save      (LogJamDoc *doc, GError **err);

/* these emit the "title-changed" signal */
gboolean logjam_doc_save_as_file (LogJamDoc *doc,
                                  const char *filename, GError **err);
gboolean logjam_doc_save_as_draft(LogJamDoc *doc,
                                  const char *title, GError **err);


gboolean logjam_doc_insert_file(LogJamDoc *doc, const char *filename, const char *encoding, GError **err);

EntryType logjam_doc_get_entry_type(LogJamDoc *doc);
gint logjam_doc_get_entry_itemid(LogJamDoc *doc);

void logjam_doc_user_changed(LogJamDoc *doc);

#define LOGJAM_TYPE_DOC (logjam_doc_get_type())
#define LOGJAM_DOC(object) (G_TYPE_CHECK_INSTANCE_CAST((object), LOGJAM_TYPE_DOC, LogJamDoc))
#define LOGJAM_DOC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), LOGJAM_TYPE_DOC, LogJamDocClass))


#endif /* __logjam_doc_h__ */
