/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include <config.h>

#include <stdlib.h>
#include <gtk/gtk.h>
#include "util.h"
#include "foldbox.h"

#include "tie.h"
#include "checkfriends.h"
#include "security.h"

/* manager:
 *  - server
 *    - user
 *      - (entries, outbox, etc?)
 */

typedef struct {
	GtkWidget *icon;
	GtkWidget *heading;
	GtkWidget *viewport;
	GtkWidget *contentcontainer;
	GtkWidget *content;
} ManagerDetail;
/* We only ever use one ManagerDetail, below, in
 * the ManagerUI struct.  The separation is only for
 * cleanliness. */

typedef struct _ManagerUI ManagerUI;

typedef char*      (*detailheading_f)(gpointer data);
typedef GtkWidget* (*detailui_f)     (ManagerUI* mui, gpointer data);
typedef void       (*detaildelete_f) (ManagerUI* mui, gpointer data);

typedef struct {
	detailheading_f get_heading;
	detailui_f      make_ui;
	detaildelete_f  delete;
} DetailFuncs;

struct _ManagerUI {
	GtkWidget *win;
	GtkWidget *treeview;

	GdkPixbuf *pb_server, *pb_user, *pb_mood;
	ManagerDetail detail;
	GtkTreeStore *store;
	
	GtkWidget *bdelete;

	gpointer selection;
	DetailFuncs *selection_funcs;

	GtkWidget *filterlabel;
};

enum {
	COL_ICON,
	COL_TEXT,
	COL_DETAILFUNCS,
	COL_DATA,
	COL_COUNT
};

static char*
user_get_heading(User *user) {
	if (user->fullname) 
		return g_strdup(user->fullname);
	return g_strdup(user->username);
}

static GtkWidget*
label_list(GList *l, gboolean friendgroups /* FIXME: hack */) {
	GtkWidget *label;
	label = lj_form_label_new("");
	if (l) {
		GString *str = g_string_sized_new(1024);

		while (l) {
			if (str->len > 0)
				g_string_append(str, ", ");
			if (friendgroups) {
				FriendGroup *fg = l->data;
				g_string_append(str, fg->name);
			} else {
				g_string_append(str, l->data);
			}
			l = l->next;
		}
		gtk_label_set_text(GTK_LABEL(label), str->str);
		g_string_free(str, TRUE);
	} else {
		gtk_label_set_markup(GTK_LABEL(label), _("<i>none</i>"));
	}
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	return label;
}

static void
run_cfmask_manager_dlg(GtkWidget *b, ManagerUI *mui) {
	User *u = mui->selection;
	guint newmask, oldmask = u->cfmask;
	newmask = custom_security_dlg_run(GTK_WINDOW(mui->win), oldmask, u);
	g_assert(mui->filterlabel);
	if (newmask != oldmask) {
		u->cfmask = newmask;
		cfmgr_mask_set(app.cfmgr, u->cfmask);
		gtk_label_set_text(GTK_LABEL(mui->filterlabel),
				u->cfmask ? _("active") : _("inactive"));
	}
}

static GtkWidget*
user_make_ui_identity(ManagerUI *mui, User *user) {
	GtkWidget *fb, *table; /*, *epassword;*/

	fb = foldbox_new_with_text(_("Identity"), TRUE, ARROW_LOC_END);
	table = lj_table_new(3, 2);
	gtk_container_set_border_width(GTK_CONTAINER(table), 0);
	gtk_table_set_col_spacings(GTK_TABLE(table), 0);
	groupedbox_pack(GROUPEDBOX(fb), table, FALSE);

	lj_table_label_content(GTK_TABLE(table), 0,
			_("User Name:"), lj_form_label_new(user->username));

	lj_table_label_content(GTK_TABLE(table), 1,
			_("_Full Name:"), lj_form_label_new(user->fullname));

	/*epassword = gtk_entry_new(); 
	gtk_entry_set_visibility(GTK_ENTRY(epassword), FALSE);
	gtk_entry_set_text(GTK_ENTRY(epassword), user->password);
	lj_table_label_content(GTK_TABLE(table), 2,
			_("_Password:"), epassword);*/
	
	return fb;
}

static void
toggle_tie_cb(GtkWidget *toggle, gboolean *opt) {
	*opt = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
}

static void
toggle_tie(GtkWidget *toggle, gboolean *opt) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle), *opt);
	g_signal_connect(G_OBJECT(toggle), "toggled",
			G_CALLBACK(toggle_tie_cb), opt);
}

static GtkWidget*
user_make_ui_loginoptions(ManagerUI *mui, User *user) {
	GtkWidget *fb, *cruser, *crpassword;

	fb = foldbox_new_with_text(_("Login Options"), FALSE, ARROW_LOC_END);

	cruser = gtk_check_button_new_with_label(_("Remember user"));
	toggle_tie(cruser, &user->remember_user);
	groupedbox_pack(GROUPEDBOX(fb), cruser, FALSE);

	crpassword = gtk_check_button_new_with_label(_("Remember password"));
	toggle_tie(crpassword, &user->remember_password);
	groupedbox_pack(GROUPEDBOX(fb), crpassword, FALSE);

	return fb;
}

static GtkWidget*
user_make_ui_pictures(ManagerUI *mui, User *user) {
	GtkWidget *fb;
	fb = foldbox_new_with_text(_("Pictures"), FALSE, ARROW_LOC_END);
	groupedbox_pack(GROUPEDBOX(fb), label_list(user->pickws, FALSE), FALSE);
	return fb;
}

static GtkWidget*
user_make_ui_friendgroups(ManagerUI *mui, User *user) {
	GtkWidget *fb;
	GtkWidget *hbcf, *lccf, *licf, *bcf;

	fb = foldbox_new_with_text(_("Friend Groups and Filtering"), FALSE, ARROW_LOC_END);
	groupedbox_pack(GROUPEDBOX(fb), label_list(user->friendgroups, TRUE), FALSE);
	
	hbcf = gtk_hbox_new(FALSE, 5);
	lccf = gtk_label_new(_("Filter for friends list monitor: "));
	licf = gtk_label_new(user->cfmask ? _("active") : _("inactive"));
	bcf  = gtk_button_new_with_label(_("Edit filter"));

	/* guard against a user with no friendgroup info */
	gtk_widget_set_sensitive(bcf, user->friendgroups != NULL);

	gtk_misc_set_alignment(GTK_MISC(licf), 0.0, 0.5);
	mui->filterlabel = licf; /* not the prettiest of hacks */

	groupedbox_pack(GROUPEDBOX(fb),   hbcf, FALSE);
	gtk_box_pack_start(GTK_BOX(hbcf), lccf, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbcf), licf, TRUE,  TRUE,  0);
	gtk_box_pack_start(GTK_BOX(hbcf), bcf,  FALSE, FALSE, 0);
	
	g_signal_connect(G_OBJECT(bcf), "clicked",
			G_CALLBACK(run_cfmask_manager_dlg), mui);

	return fb;
}

static GtkWidget*
user_make_ui(ManagerUI *mui, gpointer data) {
	GtkWidget *vbox;
	User *user = data;

	vbox  = gtk_vbox_new(FALSE, 18);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

	gtk_box_pack_start(GTK_BOX(vbox),
			user_make_ui_identity(mui, user), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox),
			user_make_ui_loginoptions(mui, user), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox),
			user_make_ui_pictures(mui, user), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox),
			user_make_ui_friendgroups(mui, user), FALSE, FALSE, 0);

	gtk_widget_show_all(vbox);

	return vbox;
}

typedef struct {
	GtkTreeIter iter;
	gpointer searchdata;
	gboolean found;
} SearchData;

static gboolean
search_model_cb(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
		gpointer data) {
	SearchData *sd = data;
	gpointer datacol;
	gtk_tree_model_get(model, iter, COL_DATA, &datacol, -1);
	if (datacol == sd->searchdata) {
		sd->found = TRUE;
		sd->iter = *iter;
		return TRUE;
	}
	return FALSE;
}

static GtkTreeIter
search_model(ManagerUI *mui, gpointer data) {
	SearchData sd;
	sd.found = FALSE;
	sd.searchdata = data;
	gtk_tree_model_foreach(GTK_TREE_MODEL(mui->store),
			search_model_cb, &sd);
	return sd.iter;
}

static void
user_delete(ManagerUI *mui, User *user) {
	GtkTreeIter it;
	it = search_model(mui, user);
}

/*
static void
add_entry_cache(ManagerUI *mui, GtkTreeStore *ts, GtkTreeIter *parent, User *user) {
	GtkTreeIter ientries;

	gtk_tree_store_append(ts, &ientries, parent);
	gtk_tree_store_set(ts, &ientries,
			COL_TEXT, _("Entries"),
			-1);
}
*/

static void
add_user(ManagerUI *mui, GtkTreeIter *parent, User *user) {
	GtkTreeIter iuser;
	/*, idrafts, ioutbox; */
	static DetailFuncs userdetail = {
		(detailheading_f)user_get_heading,
		(detailui_f)user_make_ui,
		(detaildelete_f)user_delete
	};

	gtk_tree_store_append(mui->store, &iuser, parent);
	gtk_tree_store_set(mui->store, &iuser,
			COL_ICON, mui->pb_user,
			COL_TEXT, user->username,
			COL_DETAILFUNCS, &userdetail,
			COL_DATA, user,
			-1);

	/*
	add_entry_cache(mui, ts, &iuser, user);

	gtk_tree_store_append(ts, &idrafts, &iuser);
	gtk_tree_store_set(ts, &idrafts,
			COL_TEXT, _("Drafts"),
			-1);

	gtk_tree_store_append(ts, &ioutbox, &iuser);
	gtk_tree_store_set(ts, &ioutbox,
			COL_TEXT, _("Outbox"),
			-1);
	*/
}

static char*
server_get_heading(Server *server) {
	//return g_strdup_printf("Server '%s'", server->name);
	return g_strdup(server->name);
}

static gboolean
entry_tie_cb(GtkEntry *entry, GdkEventFocus *e, char **data) {
	string_replace(data, gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1));
	return FALSE;
}
static void
entry_tie(GtkEntry *entry, char **data) {
	gtk_entry_set_text(GTK_ENTRY(entry), *data);
	g_signal_connect(G_OBJECT(entry), "focus-out-event",
			G_CALLBACK(entry_tie_cb), data);
}

static gboolean
server_rename_cb(GtkEntry *entry, GdkEventFocus *e, ManagerUI *mui) {
	Server *s = mui->selection;
	const char *newname;
	GError *err = NULL;

	newname = gtk_entry_get_text(entry);
	if (strcmp(s->name, newname) == 0)
		return FALSE;
	if (!conf_rename_server(s, newname, &err)) {
		lj_warning(GTK_WINDOW(mui->win), _("Unable to rename server: %s."),
				err->message);
		g_error_free(err);
		gtk_entry_set_text(entry, s->name);
		return FALSE;
	}

	/* XXX if Server was a gobject this would be much cleaner,
	 * as we'd just attach these widgets on the notify for the server name. */

	/* now update the tree on the left. */
	{ 
		GtkTreeSelection *ts;
		GtkTreeModel *model;
		GtkTreeIter iter;

		ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(mui->treeview));
		gtk_tree_selection_get_selected(ts, &model, &iter);
		gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
				COL_TEXT, s->name,
				-1);
	}
	/* and the title. */
	{
		char *heading;
		heading = g_strdup_printf("<span weight='bold' size='large'>%s</span>", s->name);
		gtk_label_set_markup(GTK_LABEL(mui->detail.heading), heading);
		g_free(heading);
	}
	
	return FALSE;
}

static GtkWidget*
server_make_ui(ManagerUI *mui, gpointer data) {
	Server *server = data;
	GtkWidget *vbox;
	GtkSizeGroup *sg;
	GtkWidget *ename, *eurl, *cprotocol;

	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

	vbox = gtk_vbox_new(FALSE, 6);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);

	ename = gtk_entry_new(); 
	gtk_entry_set_text(GTK_ENTRY(ename), server->name);
	g_signal_connect_after(G_OBJECT(ename), "focus-out-event",
			G_CALLBACK(server_rename_cb), mui);
	gtk_box_pack_start(GTK_BOX(vbox),
			labelled_box_new_sg(_("_Name:"), ename, sg),
			FALSE, FALSE, 0);

	eurl = gtk_entry_new(); 
	entry_tie(GTK_ENTRY(eurl), &server->url);
	gtk_widget_set_usize(eurl, 200, -1);
	gtk_box_pack_start(GTK_BOX(vbox),
			labelled_box_new_sg(_("_URL:"), eurl, sg),
			FALSE, FALSE, 0);

	cprotocol = gtk_check_button_new_with_mnemonic(
			_("_Server supports protocol version 1"));
	tie_toggle(GTK_TOGGLE_BUTTON(cprotocol), &server->protocolversion);
	gtk_box_pack_start(GTK_BOX(vbox), cprotocol, FALSE, FALSE, 0);


	return vbox;
}

static void
server_delete(ManagerUI *mui, Server *server) {
	GtkTreeIter it;

	it = search_model(mui, server);
	gtk_tree_store_remove(mui->store, &it);

	conf.servers = g_list_remove(conf.servers, server);
	g_free(server);
}

static void
add_server(ManagerUI *mui, Server *server) {
	GtkTreeIter iserver;
	GList *l;
	static DetailFuncs serverdetail = {
		(detailheading_f)server_get_heading,
		(detailui_f)server_make_ui,
		(detaildelete_f)server_delete
	};

	gtk_tree_store_append(mui->store, &iserver, NULL);
	if (server) {
		gtk_tree_store_set(mui->store, &iserver,
				COL_ICON, mui->pb_server,
				COL_TEXT, server->name,
				COL_DETAILFUNCS, &serverdetail,
				COL_DATA, server,
				-1);

		for (l = server->users; l; l = l->next) {
			add_user(mui, &iserver, l->data);
		}
	} else {
		gtk_tree_store_set(mui->store, &iserver,
				COL_ICON, mui->pb_server,
				COL_TEXT, _("Add New Server"),
				COL_DATA, NULL,
				-1);
	}
}

static void
populate_store(ManagerUI *mui) {
	GList *l;

	gtk_tree_store_clear(mui->store);
	for (l = conf.servers; l; l = l->next) {
		if (l->data)
			add_server(mui, l->data);
	}
	add_server(mui, NULL);
}

static GtkTreeStore*
make_model(ManagerUI *mui) {
	mui->store = gtk_tree_store_new(COL_COUNT, 
			/* icon */ GDK_TYPE_PIXBUF,
			/* name */ G_TYPE_STRING,
			/* detailfuncs */ G_TYPE_POINTER,
			/* data */ G_TYPE_POINTER);

	mui->pb_server = gtk_widget_render_icon(mui->treeview, "logjam-server", 
			GTK_ICON_SIZE_MENU, NULL);
	mui->pb_user = gtk_widget_render_icon(mui->treeview, "logjam-ljuser", 
			GTK_ICON_SIZE_MENU, NULL);
	mui->pb_mood = gtk_widget_render_icon(mui->treeview, "logjam-mood", 
			GTK_ICON_SIZE_MENU, NULL);

	populate_store(mui);

	return mui->store;
}

static gboolean
new_server_cb(gpointer data) {
	ManagerUI *mui = data;
	Server *s;
	s = server_new();
	s->name = g_strdup("New Server");
	s->url = g_strdup("http://hostname:port");
	conf.servers = g_list_append(conf.servers, s);
	gtk_tree_selection_unselect_all(
			gtk_tree_view_get_selection(GTK_TREE_VIEW(mui->treeview)));
	populate_store(mui);
	gtk_tree_view_expand_all(GTK_TREE_VIEW(mui->treeview));
	return FALSE;
}

static void
change_view(ManagerUI *mui, GtkTreeIter *iter) {
	char *heading;
	char buf[200] = {0};
	GdkPixbuf *pb = NULL;
	DetailFuncs *df = NULL;
	gpointer data = NULL;
	GtkRequisition requisition;

	if (iter) {
		gtk_tree_model_get(GTK_TREE_MODEL(mui->store), iter,
				COL_ICON, &pb,
				COL_DETAILFUNCS, &df,
				COL_DATA, &data,
				-1);
	}

	if (mui->detail.content) {
		gtk_widget_destroy(mui->detail.content);
		mui->detail.content = NULL;
	}

	gtk_image_set_from_pixbuf(GTK_IMAGE(mui->detail.icon), pb);

	if (df) {
		heading = df->get_heading(data);
		g_snprintf(buf, 200,
				"<span weight='bold' size='large'>%s</span>", heading);
		g_free(heading);
	}
	gtk_label_set_markup(GTK_LABEL(mui->detail.heading), buf);

	mui->selection = data;
	if (mui->selection == NULL ||
			mui->selection == conf_cur_server() ||
			mui->selection == conf_cur_user()) {
		gtk_widget_set_sensitive(mui->bdelete, FALSE);
	} else {
		gtk_widget_set_sensitive(mui->bdelete, TRUE);
	}
	mui->selection_funcs = df;

	if (df) {
		mui->detail.content = df->make_ui(mui, data);
		gtk_container_add(GTK_CONTAINER(mui->detail.contentcontainer), 
				mui->detail.content);

		/* viewport.width = content.width + 20px border */
		gtk_widget_size_request(mui->detail.content, &requisition);
		gtk_widget_set_size_request(mui->detail.viewport,
				requisition.width+20, -1);

		gtk_widget_show_all(mui->detail.content);
	}
}

static void
selection_changed_cb(GtkTreeSelection *ts, ManagerUI *mui) {
	GtkTreeModel *model;
	GtkTreeIter iter;
	gpointer data;

	if (!gtk_tree_selection_get_selected(ts, &model, &iter)) {
		change_view(mui, NULL);
	} else {
		gtk_tree_model_get(model, &iter, COL_DATA, &data, -1);
		if (data) 
			change_view(mui, &iter);
		else
			/* they clicked "add server".
			 * if we muck with the selection right here,
			 * gtk gets confused and kills itself.
			 * so we register an idle handler to
			 * handle their request immediately. */
			gtk_idle_add(new_server_cb, mui);
	}
}


static void
delete_cb(GtkWidget *b, ManagerUI *mui) {
	if (!mui->selection) {
		g_warning("delete clicked, but nothing selected?\n");
		return;
	}
	if (mui->selection == conf_cur_server()
			|| mui->selection == conf_cur_user()) {
		return;
	}
	mui->selection_funcs->delete(mui, mui->selection);
}

static GtkWidget*
make_tree(ManagerUI *mui) {
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeSelection *sel;
	GtkWidget *sw;

	mui->treeview = gtk_tree_view_new();
	gtk_tree_view_set_model(GTK_TREE_VIEW(mui->treeview),
			GTK_TREE_MODEL(make_model(mui)));
	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(mui->treeview));
	g_signal_connect(G_OBJECT(sel), "changed",
			G_CALLBACK(selection_changed_cb), mui);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(mui->treeview), FALSE);

	column = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column, "Node");

	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column, renderer, FALSE);
	gtk_tree_view_column_add_attribute(column, renderer,
			"pixbuf", COL_ICON);

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(column, renderer, FALSE);
	gtk_tree_view_column_add_attribute(column, renderer,
			"text", COL_TEXT);

	gtk_tree_view_append_column(GTK_TREE_VIEW(mui->treeview), column);

	gtk_tree_view_expand_all(GTK_TREE_VIEW(mui->treeview));

	sw = scroll_wrap(mui->treeview);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
			GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	return sw;
}

void
manager_dialog(GtkWindow *parent) {
	STACK(ManagerUI, mui);
	GtkWidget *panes;
	GtkWidget *vbox, *hbox, *sw, *viewport;

	mui->win = gtk_dialog_new_with_buttons(_("User Manager"),
			parent, GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
			NULL);
	gtk_window_set_default_size(GTK_WINDOW(mui->win), 400, 350);
	geometry_tie(mui->win, GEOM_MANAGER);

	panes = gtk_hbox_new(FALSE, 3);

	gtk_box_pack_start(GTK_BOX(panes), make_tree(mui), FALSE, FALSE, 0);

	mui->detail.heading = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(mui->detail.heading),
			_("<span weight='bold' size='large'>Manager</span>"));
	gtk_misc_set_alignment(GTK_MISC(mui->detail.heading), 0, 0.5);

	hbox = gtk_hbox_new(FALSE, 5);
	mui->detail.icon = gtk_image_new_from_pixbuf(mui->pb_server);
	gtk_box_pack_start(GTK_BOX(hbox), mui->detail.icon, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), mui->detail.heading, TRUE, TRUE, 0);
	mui->bdelete = gtk_button_new_from_stock(GTK_STOCK_DELETE);
	g_signal_connect(G_OBJECT(mui->bdelete), "clicked",
			G_CALLBACK(delete_cb), mui);
	gtk_box_pack_end(GTK_BOX(hbox), mui->bdelete, FALSE, FALSE, 0);

	mui->detail.contentcontainer = gtk_alignment_new(0, 0, 0, 1);
	sw    = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
			GTK_SHADOW_NONE);

	viewport = gtk_viewport_new(NULL, NULL);
	gtk_container_add(GTK_CONTAINER(viewport),
			mui->detail.contentcontainer);
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_NONE);
	gtk_container_add(GTK_CONTAINER(sw), viewport);
	mui->detail.viewport = viewport;

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(panes), vbox, TRUE, TRUE, 0);

	lj_dialog_set_contents(GTK_DIALOG(mui->win), panes);

	gtk_dialog_run(GTK_DIALOG(mui->win));

	conf_verify_a_server_exists();

	gtk_widget_destroy(mui->win);
	g_object_unref(G_OBJECT(mui->pb_server));
	g_object_unref(G_OBJECT(mui->pb_user));
	g_object_unref(G_OBJECT(mui->pb_mood));
}

