/* logjam - a GTK client for LiveJournal.
 * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
 *
 * vim: tabstop=4 shiftwidth=4 noexpandtab :
 */

#include "config.h" 

#include <gtk/gtk.h>
#include <glib.h>
#include <stdio.h>
#include <ctype.h>
#include <time.h>

#ifdef G_OS_WIN32
#include <direct.h> /* mkdir */
#endif

#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>

#include "md5.h"
#include "util.h"
#include "icons.h"

gboolean
ljdate_to_tm(const char *ljtime, struct tm *ptm) {
	gboolean ret = TRUE;
	if (sscanf(ljtime, "%4d-%2d-%2d %2d:%2d:%2d",
		&ptm->tm_year, &ptm->tm_mon, &ptm->tm_mday,
		&ptm->tm_hour, &ptm->tm_min, &ptm->tm_sec) < 5)
		ret = FALSE;
	ptm->tm_year -= 1900;
	ptm->tm_mon -= 1;
	ptm->tm_isdst = -1; /* -1: "the information is not available" */
	/* FIXME? convert to time_t and back, so the weekday and such get updated. 
	ttime = mktime(&tm);
	ptm = localtime(&ttime);*/
	return ret;
}

char*
tm_to_ljdate(struct tm *ptm) {
	return g_strdup_printf("%04d-%02d-%02d %02d:%02d:%02d",
		ptm->tm_year+1900, ptm->tm_mon+1, ptm->tm_mday,
		ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
}

char*
tm_to_ljdate_noseconds(struct tm *ptm) {
	return g_strdup_printf("%04d-%02d-%02d %02d:%02d",
		ptm->tm_year+1900, ptm->tm_mon+1, ptm->tm_mday,
		ptm->tm_hour, ptm->tm_min);
}

GtkWidget* lj_table_new(int rows, int cols) {
	GtkWidget *table;
	table = gtk_table_new(rows, cols, FALSE);
	return table;
}
GtkWidget* lj_table_label(GtkTable *table, int row, const char *text) {
	GtkWidget *label = gtk_label_new_with_mnemonic(text);
	gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
	gtk_table_attach(table, GTK_WIDGET(label), 
			0, 1, row, row+1, GTK_FILL, GTK_FILL, 6, 6);
	return label;
}
void lj_table_content(GtkTable *table, int row, GtkWidget *content) {
	gtk_table_attach(table, GTK_WIDGET(content), 
			1, 2, row, row+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
}
void lj_table_label_content_mne(GtkTable *table, int row, const char *text, GtkWidget *content, GtkWidget *mne) {
	GtkWidget *label;
	label = lj_table_label(table, row, text);
	lj_table_content(table, row, content);
	gtk_label_set_mnemonic_widget(GTK_LABEL(label), mne);
}
void lj_table_label_content(GtkTable *table, int row, const char *text, GtkWidget *content) {
	lj_table_label_content_mne(table, row, text, content, content);
}
void lj_table_fillrow(GtkTable *table, int row, GtkWidget *content) {
	gtk_table_attach(table, GTK_WIDGET(content), 
			0, 2, row, row+1, GTK_EXPAND|GTK_FILL, 0, 2, 2);
}

void
lj_spin_button_set(GtkSpinButton *w,
		gboolean numeric,
		gdouble min, gdouble max,
		gdouble step, gdouble page,
		gint digits) {
	g_assert(GTK_IS_SPIN_BUTTON(w));

	gtk_spin_button_set_numeric(w, numeric);
	gtk_spin_button_set_range(w, min, max);
	gtk_spin_button_set_increments(w, step, page);
	gtk_spin_button_set_digits(w, digits);
}

char* field_strdup_printf(char *fmt, char* fields[]) {
	GString *out;
	char *src = fmt;
	int index;
	char *ret;

	out = g_string_sized_new(256);

	while (*src != 0) {
		/* scan forward for a "%x" item */
		for ( ; *src != 0; src++) {
			if ((*src == '%') && (isdigit((int)*(src+1))))
				break;
			g_string_append_c(out, *src);
		}

		if (*src == 0) break;

		/* now read the %x item */
		src++;
		if (*src == 0) break; /* damnit, a % with no following char. */
		if (*src >= '0' && *src <= '9') {
			index = *src - '0';
			if (fields[index]) 
				g_string_append(out, fields[index-1]); /* 1-based index. */
		} else if (*src == '%') {
			/* %% -> % */
			g_string_append_c(out, *src);
		}
		src++;
	}
	ret = out->str;
	g_string_free(out, FALSE);
	return ret;
}

void
lj_win_set_size(GtkWindow *win, int width, int height) {
	if (width > 0) {
		if (height > 0) {
			gtk_window_set_default_size(win, width, height);
		} else {
			gtk_window_set_default_size(win, width, (int)(0.618*width));
		}
	} else if (height > 0) {
		gtk_window_set_default_size(win, (int)(1.618*height), height);
	}
}

void
lj_window_init(GtkWindow *win, GtkWindow *parent, const gchar *title, int width, int height) {
	if (parent)
		gtk_window_set_transient_for(GTK_WINDOW(win), parent);
	if (title)
		gtk_window_set_title(GTK_WINDOW(win), title);
	lj_win_set_size(GTK_WINDOW(win), width, height);
}

void
lj_dialog_set_contents_container(GtkDialog *dlg, GtkWidget *container) {
#if not_using_hig
	/* make controls line up with buttons in dialog. */
	static int width = -1;
	if (width == -1) {
		gtk_widget_style_get(GTK_WIDGET(dlg),
				"action_area_border", &width, NULL);
	}
#else
	/* hig guideline: 12 pixels. */
	int width = 12;
#endif
	gtk_container_set_border_width(GTK_CONTAINER(container), width);
	gtk_box_pack_start(GTK_BOX(dlg->vbox), container, TRUE, TRUE, 0);
	gtk_widget_show_all(dlg->vbox);
}

GtkWidget*
lj_dialog_set_contents(GtkDialog *dlg, GtkWidget *contents) {
	GtkWidget *vbox;
	vbox = gtk_vbox_new(FALSE, 5);
	if (contents)
		gtk_box_pack_start(GTK_BOX(vbox), contents, TRUE, TRUE, 0);
	lj_dialog_set_contents_container(dlg, vbox);
	return vbox;
}

GtkWidget*
lj_dialog_buttonbox_new(void) {
	return gtk_hbox_new(FALSE, 5);
}
void
lj_dialog_buttonbox_add(GtkWidget *box, GtkWidget *button) {
	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
}
GtkWidget*
lj_dialog_buttonbox_button_with_label(GtkWidget *box, const char *label) {
	GtkWidget *button;
	char buf[100];
	g_snprintf(buf, 100, "  %s  ", label);
	button = gtk_button_new_with_mnemonic(buf);
	lj_dialog_buttonbox_add(box, button);
	return button;
}
GtkWidget*
lj_dialog_buttonbox_button_from_stock(GtkWidget *box, const char *id) {
	GtkWidget *button = gtk_button_new_from_stock(id);
	lj_dialog_buttonbox_add(box, button);
	return button;
}

void
lj_dialog_set_contents_buttonbox(GtkWidget *dlg, GtkWidget *contents, GtkWidget *buttonbox) {
	GtkWidget *vbox, *hbox;;

	vbox = lj_dialog_set_contents(GTK_DIALOG(dlg), contents);

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_end(GTK_BOX(hbox), buttonbox, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	gtk_widget_show_all(vbox);
}

int
lj_confirm(GtkWindow *parent, const char *title, const char *msg) {
	GtkWidget *dlg;
	int res;

	dlg = gtk_message_dialog_new(GTK_WINDOW(parent), 0,
			GTK_MESSAGE_QUESTION,
			GTK_BUTTONS_YES_NO,
			msg);
	lj_window_init(GTK_WINDOW(dlg), parent, title, -1, -1);
	res = (gtk_dialog_run(GTK_DIALOG(dlg)) == GTK_RESPONSE_YES);
	gtk_widget_destroy(dlg);
	return res;
}

/* give a message on stderr unless running in quiet mode */
void
lj_cli_note(const char *msg, ...) {
	char buf[1024];
	va_list ap;

	if (app.quiet)
		return;

	va_start(ap, msg);
	g_vsnprintf(buf, 1024, msg, ap);
	va_end(ap);

	fprintf(stderr, "%s", buf);
}

void
string_replace(char **dest, char *src) {
	if (*dest) g_free(*dest);
	*dest = (char*)src;
}

void
gdkcolor_to_hex(GdkColor *color, char* buf) {
	g_snprintf(buf, 8, "#%02X%02X%02X",
			color->red >> 8,
			color->green >> 8,
			color->blue >> 8);
}

GtkWidget*
scroll_wrap(GtkWidget *w) {
	GtkWidget *scroll;

	scroll = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), 
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
			GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(scroll), w);
	return scroll;
}

static void
geometry_save(Geometry *geom, GtkWindow *win, GtkPaned *paned) {
	if (win && GTK_WIDGET(win)->window) {
		gtk_window_get_position(win, &geom->x, &geom->y);
		gtk_window_get_size(win, &geom->width, &geom->height);
	}
	if (paned) {
		geom->panedpos = gtk_paned_get_position(paned);
	}
}

static void
geometry_load(Geometry *geom, GtkWindow *win, GtkPaned *paned) {
	if (win && geom->width > 0) {
		gtk_window_move(win, geom->x, geom->y);
		gtk_window_set_default_size(win, geom->width, geom->height);
	}
	if (paned && geom->panedpos > 0) {
		gtk_paned_set_position(paned, geom->panedpos);
	}
}
static gboolean
geometry_win_event(GtkWindow *win, GdkEvent *e, Geometry *geom) {
#ifdef G_OS_WIN32
	/* geometry is totally broken on win32--
	 * gtk_window_get_position() returns random values. */
#else
	geometry_save(geom, win, NULL);
#endif
	return FALSE;
}
static gboolean
geometry_paned_event(GtkPaned *paned, gpointer d, Geometry *geom) {
	geometry_save(geom, NULL, paned);
	return FALSE;
}
void
geometry_tie(GtkWidget *win, GeometryType g) {
	geometry_tie_full(g, GTK_WINDOW(win), NULL);
}

void
geometry_tie_full(GeometryType g, GtkWindow *win, GtkPaned *paned) {
	Geometry *geom = &conf.geometries[g];

	/* the reference point is at the top left corner of the window itself,
	 * ignoring window manager decorations. */
	gtk_window_set_gravity(win, GDK_GRAVITY_STATIC);

	/* load the existing geometry for this window */
	geometry_load(geom, win, paned);
	/* and track the new geometry */
	g_signal_connect(G_OBJECT(win), "configure-event",
			G_CALLBACK(geometry_win_event), geom);
	if (paned)
		g_signal_connect(G_OBJECT(paned), "notify::position",
				G_CALLBACK(geometry_paned_event), geom);
}

static inline gboolean
needsescape(char c) {
	return !isalnum(c) && c != '_';
}

char*
urlencode(unsigned char *string) {
	int escapecount = 0;
	unsigned char *src, *dest;
	unsigned char *newstr;
	
	char hextable[] = { '0', '1', '2', '3', '4', '5', '6', '7',
					'8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
	
	if (string == NULL) return NULL;

	for (src = string; *src != 0; src++) 
		if (needsescape(*src)) escapecount++;
	
	newstr = g_new(char, strlen(string) - escapecount + (escapecount * 3) + 1);
	
	src = string;
	dest = newstr;
	while (*src != 0) {
		if (needsescape(*src)) {
			*dest++ = '%';
			*dest++ = hextable[*src >> 4];
			*dest++ = hextable[*src & 0x0F];
			src++;
		} else {
			*dest++ = *src++;
		}
	}
	*dest = 0;

	return newstr;
}

unsigned char* 
urldecode(char *string) {
	int destlen = 0;
	unsigned char *src, *dest;
	unsigned char *newstr;
	
	if (string == NULL) return NULL;

	for (src = string; *src != 0; src++) {
		if (*src == '%') { src+=2; } /* FIXME: this isn't robust. should check
										the next two chars for 0 */
		destlen++;
	}
	
	newstr = g_new(char, destlen + 1);
	src = string;
	dest = newstr;
	while (*src != 0) {
		if (*src == '%') {
			char h = (char)toupper(src[1]);
			char l = (char)toupper(src[2]);
			char vh, vl;
			vh = isalpha(h) ? (10+(h-'A')) : (h-'0');
			vl = isalpha(l) ? (10+(l-'A')) : (l-'0');
			*dest++ = ((vh<<4)+vl);
			src += 3;
		} else if (*src == '+') {
			*dest++ = ' ';
			src++;
		} else {
			*dest++ = *src++;
		}
	}
	*dest = 0;

	return newstr;
}

/* GLib unfortunately doesn't make up its mind over whether TRUE or
 * FALSE should denote comparison success, so its provided g_str_equal
 * is useless as a custom plugin to g_list_find_custom. */
static gboolean _str_equal(gconstpointer v1, gconstpointer v2) {
	return !g_str_equal(v1, v2);
}

static void
forget_cb(GObject *obj, gboolean *state) {
	*state = !*state;
}

void
lj_message_va(GtkWindow *parent, MessageType type, gboolean forgettable,
		const char *title, const char *message, va_list ap) {
	GtkWidget *dlg;
	GtkWidget *forget_check;
	gint msgtype = 0, buttontype = 0;
	const gchar *mtitle = NULL;
	gint res;
	gboolean forget_state;
	
	gchar ourhash[120];
	
	char fullmsg[1024];

	if (app.cli)
		return;

	g_vsnprintf(fullmsg, 1024, message, ap);
	
	{ /* compute hash of this message */
		GString *id;
		id = g_string_new(title);
		g_string_append(id, message);
		md5_hash(id->str, ourhash);
		g_string_free(id, TRUE);
	}
	
	/* do nothing if the user has asked not to view this message again */
	if (g_list_find_custom(app.quiet_dlgs, ourhash, _str_equal))
		return;
	
	switch (type) {
		case LJ_MSG_INFO:
			msgtype = GTK_MESSAGE_INFO;
			buttontype = GTK_BUTTONS_OK;
			mtitle = (gchar*)title;
			break;
		case LJ_MSG_WARNING:
			msgtype = GTK_MESSAGE_WARNING;
			buttontype = GTK_BUTTONS_CLOSE;
			mtitle = title ? (gchar*)title : _("Warning");
			break;
		case LJ_MSG_ERROR:
			msgtype = GTK_MESSAGE_ERROR;
			buttontype = GTK_BUTTONS_CLOSE;
			mtitle = title ? (gchar*)title : _("Error");
			break;
	}
	
	/* TODO: switch to lj_dialogs, which are prettier */
	dlg = gtk_message_dialog_new(parent, 0, msgtype,
			buttontype,
			fullmsg);
	gtk_window_set_title(GTK_WINDOW(dlg), title);
	gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(parent));
	
	if (forgettable) {
		forget_state = FALSE;
		forget_check = gtk_check_button_new_with_label(_("Do not show again"));
		g_signal_connect(G_OBJECT(forget_check), "toggled",
				G_CALLBACK(forget_cb), &forget_state);
		gtk_box_set_homogeneous(GTK_BOX(GTK_DIALOG(dlg)->action_area),
				FALSE); /* XXX: this doesn't work :( */
		
		/* aggressively make this dialog less ugly */
		gtk_button_box_set_layout(GTK_BUTTON_BOX(GTK_DIALOG(dlg)->action_area),
				GTK_BUTTONBOX_EDGE);
		gtk_box_set_child_packing(GTK_BOX(GTK_DIALOG(dlg)->action_area),
				((GtkBoxChild*)GTK_BOX(GTK_DIALOG(dlg)->action_area)->
				 	children->data)->widget, FALSE, FALSE, 0, GTK_PACK_END);
				
		/* force our checkbox to *really* be first */
		gtk_container_add(GTK_CONTAINER((GTK_DIALOG(dlg)->action_area)),
				forget_check);
		gtk_box_reorder_child(GTK_BOX((GTK_DIALOG(dlg)->action_area)),
				forget_check, 0);
	
		gtk_widget_show_all(GTK_DIALOG(dlg)->action_area);
	}

	res = gtk_dialog_run(GTK_DIALOG(dlg));

	/* flag this dialog for oblivion if the user didn't like it */
	if (forgettable && forget_state)
		app.quiet_dlgs = g_list_append(app.quiet_dlgs, g_strdup(ourhash));

	gtk_widget_destroy(dlg);
}

void
lj_message(GtkWindow *parent, MessageType type, gboolean forgettable,
		const char *title, const char *message, ...) {
	va_list ap;
	va_start(ap, message);
	lj_message_va(parent, type, forgettable, title, message, ap);
	va_end(ap);
}

void lj_warning(GtkWindow *parent, const char *message, ...) {
	va_list ap;

	va_start(ap, message);
	lj_message_va(parent, LJ_MSG_WARNING, FALSE, NULL, message, ap);
	va_end(ap);
}

/* utility thunk function */
void lj_messagebox(GtkWindow *parent, const char *title, const char *message) {
	lj_message(parent, LJ_MSG_INFO, FALSE, title, message);
}

gboolean
verify_dir(const char *path, GError **err) {
#ifdef G_OS_WIN32
	if (mkdir(path) < 0 && errno != EEXIST) {
#else
	/* mode 0700 so other people can't peek at passwords! */
	if (mkdir(path, 0700) < 0 && errno != EEXIST) {
#endif
		g_set_error(err, 0, 0, /* FIXME domain */
				_("Failed to create directory '%s': %s"),
				path, g_strerror(errno));
		return FALSE;
	}
	return TRUE;
}

gboolean
verify_path(char *path, int include_last, GError **err) {
	int i, len;

	len = strlen(path);
	for (i = 0; i < len; i++) {
		if (path[i] == G_DIR_SEPARATOR && i > 0) {
			path[i] = 0;
			if (!verify_dir(path, err))
				return FALSE;
			path[i] = G_DIR_SEPARATOR;
		}
	}
	if (include_last) {
		if (!verify_dir(path, err))
			return FALSE;
	}
	return TRUE;
}

void
md5_hash(const char *src, char *dest) {
	struct cvs_MD5Context context;
	unsigned char checksum[16];
	int i;

	cvs_MD5Init(&context);
	cvs_MD5Update(&context, (const unsigned char*)src, strlen(src));
	cvs_MD5Final(checksum, &context);
	for (i = 0; i < 16; i++)
		sprintf(&dest[i*2], "%02x", (unsigned int)checksum[i]);
}

void
strtoupper(char *str) {
	for (; *str; str++)
		*str = g_ascii_toupper(*str);
}

/* text sort utility function for GtkTreeModels */
gint
text_sort_func(GtkTreeModel *model, GtkTreeIter  *a, GtkTreeIter  *b,
		gpointer data) {
	gchar *ta, *tb;
	gint ret;
	gtk_tree_model_get(model, a, 0, &ta, -1);
	gtk_tree_model_get(model, b, 0, &tb, -1);
	ret = g_ascii_strcasecmp(ta, tb);
	g_free(ta);
	g_free(tb);
	return ret;
}

void
lj_widget_set_visible(GtkWidget *w, gboolean visible) {
	if (visible)
		gtk_widget_show(w);
	else
		gtk_widget_hide(w);
}

void
lj_widget_set_font(GtkWidget *w, const gchar *font_name) {
	PangoFontDescription *font_desc;
	
	font_desc = pango_font_description_from_string(font_name);
	gtk_widget_modify_font(w, font_desc);
	pango_font_description_free(font_desc);
}

gchar*
gettext_translate_func(const gchar *path, gpointer data) {
	/* gettext returns a const, but gtk wants nonconst.  *shrug*. */
	return (gchar*)gettext(path);
}

GtkWidget*
labelled_box_new_all(const char *caption, GtkWidget *w,
                     gboolean expand, GtkSizeGroup *sg, GtkWidget *mne) {
	GtkWidget *hbox, *l;

	l = gtk_label_new_with_mnemonic(caption);
	gtk_misc_set_alignment(GTK_MISC(l), 0, 0.5);
	if (sg)
		gtk_size_group_add_widget(sg, l);
	if (!mne)
		mne = w;
	if (mne)
		gtk_label_set_mnemonic_widget(GTK_LABEL(l), mne);

	hbox = gtk_hbox_new(FALSE, 12);
	gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
	if (w)
		gtk_box_pack_start(GTK_BOX(hbox), w, expand, expand, 0);
	return hbox;
}

GtkWidget*
lj_filesel_new(const gchar *title, GtkWindow *parent) {
	GtkWidget *filesel;

	filesel = gtk_file_selection_new(title);
	/* this should really be a systemwide setting, but the gtk file selector
	 * sucks. */
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
	gtk_window_set_transient_for(GTK_WINDOW(filesel), parent);
	return filesel;
}

char*
get_summary(char *subject, char *event) {
#define SUMMARY_LENGTH 50
	/* SUMMARY_LENGTH is in chars, not bytes.
	 * UTF-8 chars can be up to 6 bytes, and then 
	 * we need a bit more space to hold the "...". */

	static char buf[SUMMARY_LENGTH*6 + 5];

	if (subject && subject[0])
		return subject;

	if (event) {
		int i;
		g_utf8_strncpy(buf, event, SUMMARY_LENGTH);

		/* stop at the first newline. */
		for (i = 0; buf[i]; i++) {
			if (buf[i] == '\n') {
				buf[i] = 0;
				break; /* start "..." from this point. */
			}
		}

		if (g_utf8_strlen(event, SUMMARY_LENGTH) < SUMMARY_LENGTH)
			return buf;

		if (i == 0 || buf[i-1] != '.')
			buf[i++] = '.';
		buf[i++] = '.';
		buf[i++] = '.';
		buf[i++] = 0;
		return buf;
	}
	return NULL;
}

GtkWidget*
lj_form_label_new(char *text) {
	GtkWidget *label;
	label = gtk_label_new(text);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_label_set_selectable(GTK_LABEL(label), TRUE);
	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
	return label;
}

void
verify_utf8(char **str) {
	if (*str == NULL)
		return;  /* NULL string is valid utf-8.  :) */

	if (!g_utf8_validate(*str, -1, NULL)) {
		char *newstr;
		/* it's bad utf-8.  :(
		 * try replacing bad stuff with "."
		 * we can assume the source encoding wasn't utf-8,
		 * because if they're posting utf-8
		 * they have no excuse for bad characters. */
		g_print("Bad UTF-8; trying to recover.\n");
		newstr = g_convert_with_fallback(*str, -1, "UTF-8", "ISO-8859-1",
				".", NULL, NULL, NULL);
		if (newstr) {
			string_replace(str, newstr);
		}
	}
}


